diff --git a/man/machine-id.xml b/man/machine-id.xml
index 9bd49582fc..b40e26bbab 100644
--- a/man/machine-id.xml
+++ b/man/machine-id.xml
@@ -119,20 +119,26 @@
First Boot Semantics
- /etc/machine-id is used to decide whether a boot is the first one. The rules
+ /etc/machine-id is used to decide whether a boot is the first one. The rules
are as follows:
- If /etc/machine-id does not exist, this is a first boot. During
- early boot, systemd will write uninitialized\n to this file and overmount
- a temporary file which contains the actual machine ID. Later (after first-boot-complete.target
- has been reached), the real machine ID will be written to disk.
+ The kernel command argument systemd.condition-first-boot= may be
+ used to override the autodetection logic, see
+ kernel-command-line7.
+
+
+ Otherwise, if /etc/machine-id does not exist, this is a first
+ boot. During early boot, systemd will write uninitialized\n to
+ this file and overmount a temporary file which contains the actual machine ID. Later (after
+ first-boot-complete.target has been reached), the real machine ID will be written
+ to disk.
If /etc/machine-id contains the string uninitialized,
- a boot is also considered the first boot. The same mechanism as above applies.
+ a boot is also considered the first boot. The same mechanism as above applies.
If /etc/machine-id exists and is empty, a boot is
- not considered the first boot. systemd will still bind-mount a file
+ not considered the first boot. systemd will still bind-mount a file
containing the actual machine-id over it and later try to commit it to disk (if /etc/ is
writable).
@@ -140,8 +146,8 @@
not a first boot.
- If by any of the above rules, a first boot is detected, units with ConditionFirstBoot=yes
- will be run.
+ If according to the above rules a first boot is detected, units with
+ ConditionFirstBoot=yes will be run.
diff --git a/man/systemd.unit.xml b/man/systemd.unit.xml
index d502b6da18..ebb84e9db8 100644
--- a/man/systemd.unit.xml
+++ b/man/systemd.unit.xml
@@ -1455,15 +1455,18 @@
ConditionFirstBoot=
Takes a boolean argument. This condition may be used to conditionalize units on
- whether the system is booting up for the first time. This roughly means that /etc/
- is unpopulated (for details, see "First Boot Semantics" in
+ whether the system is booting up for the first time. This roughly means that /etc/
+ was unpopulated when the system started booting (for details, see "First Boot Semantics" in
machine-id5).
- This may be used to populate /etc/ on the first boot after factory reset, or
- when a new system instance boots up for the first time.
+ First boot is considered finished (this condition will evaluate as false) after the manager
+ has finished the startup phase.
+
+ This condition may be used to populate /etc/ on the first boot after
+ factory reset, or when a new system instance boots up for the first time.
For robustness, units with ConditionFirstBoot=yes should order themselves
before first-boot-complete.target and pull in this passive target with
- Wants=. This ensures that in a case of an aborted first boot, these units will
+ Wants=. This ensures that in a case of an aborted first boot, these units will
be re-run during the next system startup.
If the systemd.condition-first-boot= option is specified on the kernel
diff --git a/src/shared/condition.c b/src/shared/condition.c
index ffca2006c0..aa34e1e285 100644
--- a/src/shared/condition.c
+++ b/src/shared/condition.c
@@ -824,27 +824,20 @@ static int condition_test_needs_update(Condition *c, char **env) {
static int condition_test_first_boot(Condition *c, char **env) {
int r, q;
- bool b;
assert(c);
assert(c->parameter);
assert(c->type == CONDITION_FIRST_BOOT);
- r = proc_cmdline_get_bool("systemd.condition-first-boot", &b);
- if (r < 0)
- log_debug_errno(r, "Failed to parse systemd.condition-first-boot= kernel command line argument, ignoring: %m");
- if (r > 0)
- return b == !!r;
-
r = parse_boolean(c->parameter);
if (r < 0)
return r;
q = access("/run/systemd/first-boot", F_OK);
if (q < 0 && errno != ENOENT)
- log_debug_errno(errno, "Failed to check if /run/systemd/first-boot exists, ignoring: %m");
+ log_debug_errno(errno, "Failed to check if /run/systemd/first-boot exists, assuming no: %m");
- return (q >= 0) == !!r;
+ return (q >= 0) == r;
}
static int condition_test_environment(Condition *c, char **env) {
diff --git a/src/core/main.c b/src/core/main.c
index 27ed8a89ba..14a4f81452 100644
--- a/src/core/main.c
+++ b/src/core/main.c
@@ -2034,6 +2034,8 @@ static int invoke_main_loop(
}
static void log_execution_mode(bool *ret_first_boot) {
+ bool first_boot = false;
+
assert(ret_first_boot);
if (arg_system) {
@@ -2050,29 +2052,40 @@ static void log_execution_mode(bool *ret_first_boot) {
log_info("Detected architecture %s.", architecture_to_string(uname_architecture()));
- if (in_initrd()) {
- *ret_first_boot = false;
+ if (in_initrd())
log_info("Running in initrd.");
- } else {
+ else {
int r;
_cleanup_free_ char *id_text = NULL;
- /* Let's check whether we are in first boot. We use /etc/machine-id as flag file
- * for this: If it is missing or contains the value "uninitialized", this is the
- * first boot. In any other case, it is not. This allows container managers and
- * installers to provision a couple of files already. If the container manager
- * wants to provision the machine ID itself it should pass $container_uuid to PID 1. */
+ /* Let's check whether we are in first boot. First, check if an override was
+ * specified on the kernel commandline. If yes, we honour that. */
- r = read_one_line_file("/etc/machine-id", &id_text);
- if (r < 0 || streq(id_text, "uninitialized")) {
- if (r < 0 && r != -ENOENT)
- log_warning_errno(r, "Unexpected error while reading /etc/machine-id, ignoring: %m");
+ r = proc_cmdline_get_bool("systemd.condition-first-boot", &first_boot);
+ if (r < 0)
+ log_debug_errno(r, "Failed to parse systemd.condition-first-boot= kernel commandline argument, ignoring: %m");
- *ret_first_boot = true;
- log_info("Detected first boot.");
- } else {
- *ret_first_boot = false;
- log_debug("Detected initialized system, this is not the first boot.");
+ if (r > 0)
+ log_full(first_boot ? LOG_INFO : LOG_DEBUG,
+ "Kernel commandline argument says we are %s first boot.",
+ first_boot ? "in" : "not in");
+ else {
+ /* Second, perform autodetection. We use /etc/machine-id as flag file for
+ * this: If it is missing or contains the value "uninitialized", this is the
+ * first boot. In other cases, it is not. This allows container managers and
+ * installers to provision a couple of files in /etc but still permit the
+ * first-boot initialization to occur. If the container manager wants to
+ * provision the machine ID it should pass $container_uuid to PID 1. */
+
+ r = read_one_line_file("/etc/machine-id", &id_text);
+ if (r < 0 || streq(id_text, "uninitialized")) {
+ if (r < 0 && r != -ENOENT)
+ log_warning_errno(r, "Unexpected error while reading /etc/machine-id, ignoring: %m");
+
+ first_boot = true;
+ log_info("Detected first boot.");
+ } else
+ log_debug("Detected initialized system, this is not the first boot.");
}
}
@@ -2092,9 +2105,9 @@ static void log_execution_mode(bool *ret_first_boot) {
arg_action == ACTION_TEST ? " test" : "",
getuid(), strna(t), systemd_features);
}
-
- *ret_first_boot = false;
}
+
+ *ret_first_boot = first_boot;
}
static int initialize_runtime(
@@ -2134,7 +2147,7 @@ static int initialize_runtime(
(void) os_release_status();
(void) hostname_setup(true);
/* Force transient machine-id on first boot. */
- machine_id_setup(NULL, first_boot, arg_machine_id, NULL);
+ machine_id_setup(NULL, /* force_transient= */ first_boot, arg_machine_id, NULL);
(void) loopback_setup();
bump_unix_max_dgram_qlen();
bump_file_max_and_nr_open();